%! -*- coding:utf-8; mode:latex; -*-

\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{tgi-tut}[2013/10/01]
\LoadClass[titlepage,oneside]{article}



% ---------------------------------- %
%  Package Loading and Simple Setup  %
% ---------------------------------- %


\RequirePackage{etoolbox}


% page geometry

\RequirePackage{geometry}
\RequirePackage{multicol}
\geometry{%
  papersize={128mm,96mm},%
  top=0.03\paperheight,%
  bottom=0.07\paperheight,%
  left=0.05\paperwidth,%
  right=0.05\paperwidth,%
}
\pagestyle{empty}


% fonts

\RequirePackage{xltxtra}
\RequirePackage[sfdefault]{libertine}
\RequirePackage[libertine]{newtxmath}
\setmonofont[Mapping=tex-text]{Latin Modern Mono}
\RequirePackage{amsmath,amsfonts}
\RequirePackage{pifont}
\setlength{\parindent}{\z@}
\setlength{\parskip}{2ex plus 1fil}


% language

\RequirePackage{polyglossia}
\setmainlanguage[spelling=new]{german}


% graphics

\RequirePackage{graphicx}
\RequirePackage{tikz}
\usetikzlibrary{positioning,shapes}


% tables

\RequirePackage{tabularx}


% listings

\RequirePackage{listings}
\lstset{
  basicstyle={\small},%
  mathescape=true,%
  columns=fullflexible,%
  morecomment=[l]{;;},%
  morekeywords={%
    ROUTINE,BEGIN,END,%
    FOR,FOREACH,DO,DONE,TO,AND,OR,IF,THEN,ELSE,FI,CALL,RETURN,TRUE,FALSE%
  },
}


% hyperlinks

\RequirePackage{hyperref}
\hypersetup{%
  pdfborder={0 0 0},%
}


% -------- %
%  Slides  %
% -------- %


%% .. rubric:: Synopsis
%%
%%     \begin{slide}[TITLE]
%%         MATERIAL...
%%     \end{slide}
%%
%%
%% .. rubric:: Description
%%
%% Typeset the MATERIAL on a presentation slide (unless it doesn't fit on one
%% page in which case it will be split).  Optionally typeset TITLE as an
%% unnumbered level-1 section.
%%
\newenvironment{slide}[1][]{%
  \ifblank{#1}{}{%
    \section*{#1}%
  }%
}{%
  \clearpage%
}



% ------------------------------------------ %
%  Title Page and According Preamble Macros  %
% ------------------------------------------ %


% This document class does not provide an ``\author`` macro; use ``\tutor``
% instead.
\let\author\undefined
\let\@author\undefined

%% .. rubric:: Synopsis
%%
%%     \title{TITLE}
%%
%%
%% .. rubric:: Description
%%
%% Set the title of the class to be TITLE.
%%
\let\title\@undefined
\let\@title\@undefined
\newcommand*{\title}[1]{\newcommand*{\@title}{#1}}

%% .. rubric:: Synopsis
%%
%%     \tutor{SEMESTER}
%%
%%
%% .. rubric:: Description
%%
%% Set the semester of the class to be SEMESTER.  This should probably be
%% something like “winter 1976”.
%%
\let\semester\@undefined
\let\@semester\@undefined
\newcommand*{\semester}[1]{\newcommand*{\@semester}{#1}}

%% .. rubric:: Synopsis
%%
%%     \tutorial{TUTORIAL}
%%
%%
%% .. rubric:: Description
%%
%% Set the tutorial to be TUTORIAL.  This should probably be something like
%% “tutorial 42”.
%%
\let\tutorial\@undefined
\let\@tutorial\@undefined
\newcommand*{\tutorial}[1]{\newcommand*{\@tutorial}{#1}}

%% .. rubric:: Synopsis
%%
%%     \tutor{NAME}
%%
%%
%% .. rubric:: Description
%%
%% Set the name of the tutor to be NAME.
%%
\let\tutor\@undefined
\let\@tutor\@undefined
\newcommand*{\tutor}[1]{\newcommand*{\@tutor}{#1}}

%% .. rubric:: Synopsis
%%
%%     \date{DAY}
%%
%%
%% .. rubric:: Description
%%
%% Set the date for the this set of slides to be DAY.  This should probably be
%% something like ``\today``.
%%
\let\date\@undefined
\let\@date\@undefined
\newcommand*{\date}[1]{\newcommand*{\@date}{#1}}

%% .. rubric:: Synopsis
%%
%%     \maketitle
%%
%%
%% .. rubric:: Description
%%
%% Insert a ``\slide`` with the title page.  This will not work if the preamble
%% macros ``\title``, ``\semester``, ``\tutorial``, ``\tutor`` ans ``\date``
%% have not been called correctly.
%%
%%
%% .. rubric:: Files
%%
%% **./titlepicture.jpg**
%%     If this file exists, it should be a picture of reasonable size with an
%%     aspect ratio of 8:3.  It is inserted as the title picture.  If the file
%%     does not exist, a warning text is inserted instead.
%% **./titlepicture-copyright.tex**
%%     If this file exists and ``./titlepicture.jpg`` exists too, the contents
%%     of ``./titlepicture-copyright.tex`` are ``\input`` and placed as a
%%     copyright notice at the bottom of the title slide.
%%
\renewcommand{\maketitle}{%
  \begin{slide}
    \scshape%
    \addfontfeatures{Numbers={OldStyle,Proportional}}%
    \begingroup\LARGE\@title\endgroup%
    \par\bigskip%
    \begin{tabular}{@{}lcl@{}}%
      \@tutorial & \quad & \@semester\\[1ex]%
      \@tutor    & \quad & \@date\\%
    \end{tabular}%
    \par\vfill%
    \begin{tikzpicture}[remember picture, overlay]%
      \node [inner sep = 0pt, anchor = south] at (current page.south) {%
        \IfFileExists{./titlepicture.jpg}{%
          \includegraphics[%
            width=\paperwidth,%
            height=0.5\paperheight%
          ]{./titlepicture.jpg}%
        }{%
          \begin{minipage}{\textwidth}%
            \normalfont\small%
            Warning: The file \file{titlepicture.jpg} was not found.%
            \par\bigskip%
          \end{minipage}%
        }%
      };%
      \node [anchor = south] at (current page.south) {%
        \begin{minipage}{\textwidth}%
          \IfFileExists{./titlepicture.jpg}{%
            \IfFileExists{./titlepicture-copyright.tex}{%
              \normalfont\tiny%
              \input{./titlepicture-copyright.tex}%
            }{%
              % no copyright notice
            }%
          }{%
            % no copyright notice
          }%
        \end{minipage}%
      };
    \end{tikzpicture}%
  \end{slide}%
}



% --------------------------------------- %
%  Miscelanneous Macros and Environments  %
% --------------------------------------- %


%% .. rubric:: Synopsis
%%
%%     \begin{centerxy}
%%         MATERIAL...
%%     \end{centerxy}
%%
%%
%% .. rubric:: Description
%%
%% Typeset the MATERIAL horizonally and vertically ceneterd.  Vertical
%% centering is done with the force of one ``\fill``.  This environment is
%% especially useful for typesetting things like “thank you for your attention”
%% on a single slide.
%%
\newenvironment{centerxy}{%
  \vspace*{1\fill}%
  \begin{center}%
}{%
  \end{center}%
  \vspace*{1\fill}%
}

%% .. rubric:: Synopsis
%%
%%     \YesOrNo{BOOL}{LEMMA}{EXPLANATION}
%%
%%
%% .. rubric:: Description
%%
%% Produce two slides.  The first slide shows LEMMA.  The second slide marks
%% LEMMA as either true or false (depending on BOOL which must be the string
%% ``true`` or ``false``) and shows EXPLANATION.
%%
\newbool{tgi@yesorno}
\newcommand*{\tgi@yes}{\textcolor{green}{\ding{51}}}
\newcommand*{\tgi@no}{\textcolor{red}{\ding{55}}}
\newcommand{\YesOrNo}[3]{%
  \setbool{tgi@yesorno}{#1}
  \begin{slide}[Wahr oder falsch?]
    \begin{itemize}
      \item[\null] #2
    \end{itemize}
  \end{slide}
  \begin{slide}[Wahr oder falsch?]
    \begin{itemize}
      \item[\ifbool{tgi@yesorno}{\tgi@yes}{\tgi@no}] #2
    \end{itemize}
    #3
  \end{slide}
}


%% .. rubric:: Synopsis
%%
%%     \mathemph{EXPRESSION}
%%
%%
%% .. rubric:: Description
%%
%% Emphasize EXPRESSION while in math mode.
%%
\providecommand*{\mathemph}[1]{\textcolor{purple}{#1}}

%% .. rubric:: Synopsis
%%
%%     \strong{TEXT}
%%
%%
%% .. rubric:: Description
%%
%% Typeset TEXT using strong emphasis.
%%
\providecommand*{\strong}[1]{\textcolor{purple}{\textbf{#1}}}

%% .. rubric:: Synopsis
%%
%%     \code{CODE}
%%
%%
%% .. rubric:: Description
%%
%% Typeset CODE as soucre code.
%%
\providecommand*{\code}[1]{\mbox{\texttt{#1}}}

%% .. rubric:: Synopsis
%%
%%     \file{PATH}
%%
%%
%% .. rubric:: Description
%%
%% Typeset PATH as a file name.
%%
\let\file\code

%% .. rubric:: Synopsis
%%
%%     \person{NAME}
%%
%%
%% .. rubric:: Description
%%
%% Typeset NAME as somebody's name.
%%
\providecommand*{\person}[1]{\textsc{#1}}

%% .. rubric:: Synopsis
%%
%%     \literal{LITERAL}
%%
%%
%% .. rubric:: Description
%%
%% Typeset LITERAL as a string (character) literal.  This can be used in text
%% or math mode.
%%
\newcommand{\literal}[1]{\texttt{#1}}

%% .. rubric:: Synopsis
%%
%%     \begin{SheetSummary}{SHEET}{SUBMISSIONS}
%%         \exercise[MEAN]{MIN}{1Q}{MEDIAN}{3Q}{MAX}
%%         ...
%%     \end{SheetSummary}
%%
%%
%% .. rubric:: Description
%%
%% Produce a slide that shows a summary of exercise sheet number SHEET with a
%% total of SUBMISSIONS valid submissions.  For each exercise on that sheet (in
%% order),  include an ``\exercise`` macro that specifies (in that oder)
%%
%% * optionally: the mean
%% * the minimum
%% * the 1st quartile
%% * the median
%% * the 3rd quartile
%% * and the maximum
%%
%% number of points a student was graded for this exercise.  All numbers may be
%% float values.  The mean should be specified using fixed-width precision.  It
%% will be assumed that an exercise is always graded between 0 and 4 points.
%% Do not include any other material inside this environment.
%%
%% This code can be produced automatically from the student database by the
%% ``admin.py`` script.
%%
\newcounter{tgi@exercise}
\newlength{\tgi@pw}
% #1  exercise sheet number
% #2  number of valid submissions
\newenvironment{SheetSummary}[2]{%
  \setcounter{tgi@exercise}{0}%
  \setlength{\tgi@pw}{0.25\textwidth}%
  \footnotesize%
  % [#1]  mean
  %  #2   minimum
  %  #3   1st quartile
  %  #4   median
  %  #5   3rd quartile
  %  #6   maximum
  \newcommand{\exercise}[6][-1]{%
    \stepcounter{tgi@exercise}%
    \leavevmode%
    \begin{tikzpicture}[%
        span/.style = {%
          draw = cyan,%
        },%
        halfspan/.style = {%
          draw = cyan,%
          fill = cyan!10,%
          very thin,%
        },%
        median/.style = {%
          draw = cyan,%
          very thick,%
        },%
        mean/.style = {%
          circle,%
          inner sep = 0pt,%
          fill = cyan,%
          minimum width = 1mm,%
        },%
      ]%
      \path[use as bounding box] (0\tgi@pw,-0.4) rectangle (4\tgi@pw,0.4);%
      \node[anchor = south west] at (0,0) {Aufgabe~\thetgi@exercise};%
      \coordinate (aux@lower) at (0,-0.1);%
      \coordinate (aux@upper) at (0,0.1);%
      \coordinate (mean) at (##1\tgi@pw,0);%
      \coordinate (min) at (##2\tgi@pw,0);%
      \coordinate (q1) at (##3\tgi@pw,0);%
      \coordinate (median) at (##4\tgi@pw,0);%
      \coordinate (q3) at (##5\tgi@pw,0);%
      \coordinate (max) at (##6\tgi@pw,0);%
      \draw[span] (min) -- (max);%
      \draw[span] (min|-aux@lower) -- (min|-aux@upper);%
      \draw[span] (max|-aux@lower) -- (max|-aux@upper);%
      \draw[halfspan] (q1|-aux@lower) rectangle (q3|-aux@upper);%
      \draw[median] (median|-aux@lower) -- (median|-aux@upper);%
      \ifdimcomp{##1pt}{<}{0pt}{}{%
        \node [mean] at (mean) {};%
        \node [above = 1mm of mean] {$\bar{x}=##1$};%
      }%
    \end{tikzpicture}%
    \par%
  }%
  \begin{slide}[#1.~Übungsblatt (#2 gültige Abgaben)]%
}{%
    \leavevmode%
    \begin{tikzpicture}%
      \path[use as bounding box] (0\tgi@pw,-0.4) rectangle (4\tgi@pw,0.4);%
      \draw (0\tgi@pw,0) -- (4\tgi@pw,0);%
      \foreach \i in {0,...,4} {%
        \draw (\i\tgi@pw,-0.1) -- +(0,0.2);%
        \node[anchor = north] at (\i\tgi@pw,-0.2) {\i};%
      }%
      \node at (2\tgi@pw,-0.8) {Punkte};%
    \end{tikzpicture}%
    \vspace{-1cm}%
  \end{slide}%
}


% mathematical functions

\newcommand{\prob}[1]{\mathrm{Pr}\left[#1\right]}


% miscelaneous

\newcommand*{\textsmall}[1]{{\small#1}}
